﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using Holoville.HOTween;
using Holoville.HOTween.Plugins;

public class LevelManager:MonoBehaviour {
	public static LevelManager instance;

	// Current level
	int currentLevel = 0;
	[HideInInspector] public bool started = false;
	private bool armPaused = true;

	// Credits
	public GameObject[] credits;

	// Start Button
	public GameObject startButton;

	// Sound
	public AudioClip successSound;
	public AudioClip failSound;
	public AudioClip victorySong;
	public AudioSource audioSrc;
	public AudioLowPassFilter audioFilter;

	// Pulsing
	public Light lightSource;
	public Renderer sunRenderer;
	public Color colorOne;
	public Color colorTwo;
	//private float intensity = 0f;
	//private float intensityAverage = 0f;
	private float timer = 0f;
	private float[] freqData = new float[1024];

	// Victory
	public ParticleSystem victoryParticles;

	// Animation
	public Animation anim;
	private AnimationState animState;
	public AnimationCurve easeCurve;
	public AnimationCurve firstShapeFlyInScaleCurve;

	// Constellation texture
	public GameObject constellationTexture;

	// Reference to hand
	public Transform characterHand;

	// Container for everything
	public Transform allContainer;

	// References to shape prefabs
	public GameObject[] shapePrefabs;

	// Particle system prefab
	public GameObject particlesPrefab;

	// Material for answer shape
	public Material hiddenMaterial;

	// Camera position
	public Transform camTransform;
	public Transform camPos;

	// Spawned shapes
	private int shapeOffset;
	private int targetIndex;
	private GameObject[] shapeContainers;
	public GameObject[] shapes;
	private ParticleSystem[] particleSystems;
	private Vector3 constellationOriginPos = new Vector3(-0.6f, 3.6f, 5.5f);
	private Vector3 originPos = new Vector3(-3f, 2f, 5.5f);
	private Vector3 offsetPos = new Vector3(3f, 0f, 0f);
	private Vector3 answerOffsetPos = new Vector3(-0.35f, 0.1f, 0.33f);

	// Target shape
	[HideInInspector] public GameObject targetShape;

	// Spawn target shapes
	public void Awake() {
		instance = this;
		anim.Play("RaiseHand");
		anim.wrapMode = WrapMode.ClampForever;
		animState = anim["RaiseHand"];
		animState.speed = 0f;
		shapeOffset = 0; // STARTING LEVEL (multiply by 3)
		Crosshair.selected = true;
	
		allContainer.transform.position = originPos + (Vector3.up * 10f);

		shapeContainers = new GameObject[3];
		shapes = new GameObject[3];
		particleSystems = new ParticleSystem[3];
	}

	public void StartPlaying() {
		audioSrc.PlayOneShot(successSound);
		Vector3 tempPos = startButton.transform.position;
		Destroy(startButton);
		started = true;
		HOTween.To(allContainer.transform, 9f, new TweenParms().Prop("position", originPos).Ease(EaseType.EaseOutSine).OnComplete(Done));
		SpawnShapes();

		// Scale shapes in
		for(int i = 0; i < 3; i++) {
			shapes[i].transform.localScale = Vector3.zero;
			HOTween.To(shapes[i].transform, 9f, new TweenParms().Prop("localScale", Vector3.one).Ease(EaseType.EaseInExpo));
		}

		// Fly in target shape for first level
		Vector3[] path = new Vector3[2];

		path[0] = targetShape.transform.position + new Vector3(1f, 3f, 2f);
		path[1] = targetShape.transform.position;
		
		targetShape.transform.localScale = Vector3.zero;
		targetShape.transform.position = tempPos;

		HOTween.To(targetShape.transform, 5f, new TweenParms().Prop("localScale", Vector3.one * 0.1f).Ease(firstShapeFlyInScaleCurve));
		HOTween.To(targetShape.transform, 5f, new TweenParms().Prop("position", new PlugVector3Path(path, PathType.Curved)).Ease(EaseType.EaseInSine).OnComplete(EnableArm));
	}

	public void SpawnShapes() {
		GameObject particles;
		SphereCollider col;
		Vector3 pos;
		CrosshairTarget ct;

		targetIndex = 0;

		// Randomize index for second and third level
		if(shapeOffset < 9 && shapeOffset != 0) {
			targetIndex = Random.Range(0, 2);
		}

		// Randomize positions
		List<Vector3> positions = new List<Vector3>(3);
		for(int j = 0; j < 3; j++) {
			pos = originPos + (offsetPos * j);
			pos.z += -Mathf.Abs(pos.x) * 0.75f;
			pos += allContainer.transform.position - originPos;
			positions.Add(pos);
		}

		// Spawn shapes
		int i;
		int k;
		for(int j = 0; j < 3; j++) {
			i = currentLevel + j;
			// Randomly select position and remove
			//if(j == 0) {
			//	k = 1; // TODO - comment
			//} else {
				k = Random.Range(0, positions.Count);
			//}
			pos = positions[k];
			positions.RemoveAt(k);
			// Container
			shapeContainers[i] = new GameObject(shapePrefabs[i + shapeOffset].name);
			shapeContainers[i].transform.position = pos;
			// Collider
			col = shapeContainers[i].AddComponent<SphereCollider>();
			col.radius = 1.5f;
			col.isTrigger = true;
			// Shape
			shapes[i] = (GameObject) Instantiate(shapePrefabs[i + shapeOffset], shapeContainers[i].transform.position, Random.rotation);
			shapes[i].name = shapePrefabs[i + shapeOffset].name;
			shapes[i].transform.parent = shapeContainers[i].transform;
			shapes[i].transform.localPosition = Vector3.zero;
			shapes[i].renderer.material = hiddenMaterial;
			shapes[i].layer = LayerMask.NameToLayer("IgnoredByMainCam");
			// Shape rotation
			shapes[i].AddComponent<RotateWithCamera>().camTransform = camTransform;

			// Particles
			particles = (GameObject) Instantiate(particlesPrefab, shapeContainers[i].transform.position, Quaternion.identity);
			particles.name = shapePrefabs[i + shapeOffset].name + "_Particles";
			particles.transform.parent = shapeContainers[i].transform;
			particles.transform.localPosition = Vector3.zero;
			// Crosshair target
			ct = shapeContainers[i].AddComponent<CrosshairTarget>();
			ct.particles = particles.GetComponent<ParticleSystem>();
			ct.shape = shapes[i];
			particleSystems[i] = ct.particles;

			// Tween in
			shapeContainers[i].transform.localScale = Vector3.zero;
			HOTween.To(shapeContainers[i].transform, 0.75f, new TweenParms().Prop("localScale", Vector3.one).Ease(EaseType.EaseOutBounce));

			// Answer/target shape
			if(i == targetIndex) {
				targetShape = (GameObject) Instantiate(shapePrefabs[i + shapeOffset], Vector3.zero, Random.rotation);
				targetShape.name = shapePrefabs[i + shapeOffset].name;
				targetShape.transform.parent = characterHand;
				targetShape.transform.localPosition = answerOffsetPos;
				targetShape.transform.localScale = Vector3.one * 0.1f;
				targetShape.layer = LayerMask.NameToLayer("IgnoredByTopCam");
				targetShape.AddComponent<RotateAuto>().speed = new Vector3(35f, 35f, 0f);
			}

			// Add to all container
			shapeContainers[i].transform.parent = allContainer;
		}
	}

	public void Update() {
		float t = camTransform.localEulerAngles.x;
		t -= 23f;
		if(t < 0f) {
			t = 0f;
		}
		if(t > 300f) {
			t = 0f;
		}
		if(t > 10f) {
			t = 10f;
		}

		float tm = t / 10f;

		if(armPaused) {
			tm = 0f;
		}

		animState.time = iTween.FloatUpdate(animState.time, tm * animState.clip.length, 3f);
		if(targetShape != null) {
			iTween.ScaleUpdate(targetShape, Vector3.one * 0.1f * (1.25f + tm), 1f);
		}

		timer += Time.deltaTime * 2f;

		//Debug.DrawLine(Vector3.right * timer, Vector3.right * timer + Vector3.up * GetFrequencyValue() * 500f, Color.yellow, 1000f);
		//Debug.DrawLine(Vector3.right * timer, Vector3.right * timer + Vector3.up * intensityAverage * 500f, Color.red, 1000f);

		float fl = easeCurve.Evaluate(timer);
		Color c = Color.Lerp(colorOne, colorTwo, fl / 2f);

		lightSource.intensity = 0.5f + fl * 0.5f;
		sunRenderer.material.color = c;
	}

	public float GetFrequencyValue() {
		float ret = 0f;
		audioSrc.GetSpectrumData(freqData, 2, FFTWindow.Blackman);

		for(int i = 0; i < freqData.Length / 2; i++) {
			ret += freqData[i];
			//Debug.DrawLine(Vector3.right * 0.01f * i, Vector3.right * 0.01f * i + Vector3.up * freqData[i] * 5f, Color.white, 0f);
		}
		ret /= (freqData.Length / 2);

		return ret;
	}

	public void HideAnswerShapes(bool all) {
		for(int i = 0; i < 3; i++) {
			// Non answers
			if(all || i != targetIndex) {
				// Particles drop
				particleSystems[i].gravityModifier = 1f;
				particleSystems[i].emissionRate = 0f;
				// Shape shrink
				HOTween.To(shapes[i].transform, 0.75f, new TweenParms().Prop("localScale", Vector3.zero).Ease(EaseType.EaseInExpo));
			}
		}

		float delay = 1f;

		// Got it right
		if(! all) {
			audioSrc.PlayOneShot(successSound);
			shapeOffset += 3;
			delay = 3f;
		} else {
			audioSrc.PlayOneShot(failSound, 0.5f);
		}

		StartCoroutine(DestroyShapes(delay, ! all));
	}

	public IEnumerator DestroyShapes(float delay, bool correct) {
		GameObject constellation = null;
		GameObject finalShape = null;

		yield return new WaitForSeconds(delay);

		for(int i = 0; i < 3; i++) {
			if(i != targetIndex || shapeOffset < 12) { // T
				HOTween.To(shapes[i].transform, 0.75f, new TweenParms().Prop("localScale", Vector3.zero).Ease(EaseType.EaseInExpo));
			}
			particleSystems[i].enableEmission = false;
		}
		HOTween.To(targetShape.transform, 0.75f, new TweenParms().Prop("localScale", Vector3.zero).Ease(EaseType.EaseInExpo));
		targetShape = null;

		yield return new WaitForSeconds(1.25f);
		
		for(int i = 0; i < 3; i++) {
			if(i != targetIndex || shapeOffset < 12) {
				Destroy(shapeContainers[i]);
			} else {
				// Unparent
				finalShape = shapes[i];
				finalShape.transform.parent = null;
				Destroy(finalShape.GetComponent<RotateWithCamera>());
				Destroy(shapeContainers[i]);
			}
		}
		Destroy(targetShape);

		// Fly up to heaven
		if(finalShape != null && correct) {
			string n = finalShape.name;
			finalShape.name = n + "_fadeout";
			GameObject finalShapeDestination = GameObject.Find(n);

			// Move shape to constelation
			finalShape.AddComponent<RotateAuto>().speed = new Vector3(Random.Range(3f, 5f) * 50f, Random.Range(3f, 5f) * 50f, Random.Range(3f, 5f) * 50f);
			HOTween.To(finalShape.transform, 4f, new TweenParms().Prop("position", finalShapeDestination.transform.position).Ease(EaseType.EaseOutQuad));
			HOTween.To(finalShape.transform, 3.5f, new TweenParms().Prop("localScale", Vector3.zero).Ease(EaseType.Linear));

			yield return new WaitForSeconds(2.75f);

			// Scale up target shape
			HOTween.To(finalShapeDestination.transform, 1f, new TweenParms().Prop("localScale", Vector3.one).Ease(EaseType.EaseOutBounce));

			// Destroy final shape, storing constellation
			constellation = finalShapeDestination.transform.parent.gameObject;
			Destroy(finalShape);

			yield return new WaitForSeconds(1f);
		}


		if(shapeOffset < shapePrefabs.Length) {
			// Next shape
			SpawnShapes();

			yield return new WaitForSeconds(1f);
			Crosshair.selected = false;
		} else {
			armPaused = true;

			audioSrc.clip = victorySong;
			audioSrc.Play();

			victoryParticles.emissionRate = 30f;
			victoryParticles.gameObject.SetActive(false);
			victoryParticles.gameObject.SetActive(true);

			// Done with this shape, animate the big shape to us
			if(constellation != null) {
				Vector3 origEulerAngles = constellation.transform.eulerAngles;

				HOTween.To(constellationTexture.renderer.material, 1f, new TweenParms().Prop("color", new Color(1f, 1f, 1f, 1f)).Ease(EaseType.Linear));
				HOTween.To(constellation.transform, 3f, new TweenParms().Prop("position", new Vector3(-1.2f, 8.6f, 18.8f)).Ease(EaseType.EaseInOutQuad));
				HOTween.To(constellation.transform, 3f, new TweenParms().Prop("eulerAngles", new Vector3(18.7f, 181.53f, 351.2f)).Ease(EaseType.EaseInOutQuad));
				HOTween.To(constellation.transform, 3f, new TweenParms().Prop("localScale", Vector3.one * 1.9f).Ease(EaseType.EaseInOutQuad));

				yield return new WaitForSeconds(6f);
				
				HOTween.To(constellationTexture.renderer.material, 2f, new TweenParms().Prop("color", new Color(1f, 1f, 1f, 0f)).Ease(EaseType.Linear));
				HOTween.To(constellation.transform, 5f, new TweenParms().Prop("position", constellationOriginPos).Ease(EaseType.EaseInOutQuad));
				HOTween.To(constellation.transform, 5f, new TweenParms().Prop("eulerAngles", origEulerAngles).Ease(EaseType.EaseInOutQuad));
				HOTween.To(constellation.transform, 5f, new TweenParms().Prop("localScale", Vector3.one).Ease(EaseType.EaseInOutQuad));

				yield return new WaitForSeconds(5f);

				constellation.AddComponent<RotateAuto>().speed = new Vector3(0f, 10f, 0f);

				// Credits
				HOTween.To(credits[0].transform, 1f, new TweenParms().Prop("localScale", Vector3.one * 10f).Ease(EaseType.EaseOutBounce));
				HOTween.To(credits[1].transform, 1f, new TweenParms().Prop("localScale", Vector3.one * 13f).Ease(EaseType.EaseOutBounce));
			}
		}
	}

	public void Done() {
		StartCoroutine(DoneDelay());
	}

	public void EnableArm() {
		armPaused = false;
	}

	public IEnumerator DoneDelay() {
		yield return new WaitForSeconds(1.4f);
		Crosshair.selected = false;
	}
}
